/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package uoc.carlesfigueras.passwordvault;

import java.sql.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class represents a connection to the sqlite database
 * 
 * @author Carles Figueras Garcia
 */
public class ConnectorDB {
    
    private static int index; // The connection index. You may use if you get concurrent connections
    private Connection connection;
    
    /**
     * Constructor
     */
    public ConnectorDB() {
        if (index != 1) {
            try {
                Class.forName("org.sqlite.JDBC");
                connection = DriverManager.getConnection("jdbc:sqlite:./data/passwordvault.db");
                //System.out.println("Abans de connectar...");
                //connection = DriverManager.getConnection("jdbc:sqlite:31.170.167.96/public_html/passwordvault.db");
                index = 1;
            } catch (SQLException | ClassNotFoundException ex) {
                Logger.getLogger(ConnectorDB.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    
    /**
     * Gets the connection
     * 
     * @return the connection
     */
    public Connection getConnection() {               
        return connection;
    }
    
        
    /**
     * Creates an user into the database
     * 
     * @param password the password to create
     * @return true if all gones well
     * @throws java.lang.Exception
     */
    public boolean crearUsuari(User password) throws Exception {
        boolean creat;
        
        PreparedStatement st = connection.prepareStatement("insert into users values (?,?,?,?);");
        st.setString(2, password.getUser());
        st.setString(3, password.toString());
        st.setString(4, password.getSalt());
        connection.setAutoCommit(false);
        creat = st.execute();
        connection.setAutoCommit(true);

        return creat;
    }
    
    /**
     * Gets an user from the database
     * 
     * @param username the username to get
     * @return the requested user
     * @throws Exception 
     */
    public User readUser(String username) throws Exception {
        PreparedStatement st = connection.prepareStatement("select * from users where user = ?");
        st.setString(1, username);
        ResultSet rs = st.executeQuery();
        
        User user = new User(rs.getString("user"), rs.getString("password"), rs.getString("salt"));
        return user;
    }
    
    /**
     * This method makes all the process to login the application
     * @param p the login user
     * @return true if the login is successful
     * @throws java.lang.Exception
     */
    public boolean login(User p) throws Exception {
        PreparedStatement st = connection.prepareStatement("select user_id,password,salt from users where user = ?");
        st.setString(1, p.getUser());
        ResultSet rs = st.executeQuery();
        p.setId(rs.getInt("user_id"));
        
        return p.checkPassword(rs.getString("password")) == 0;
  
    }
    
    /**
     * Check if exist an user
     * 
     * @param user the username to check
     * @return true if the user exists and false if not
     */
    public boolean checkUserCreated(String user) {
        boolean creat;
        ResultSet rs; 
        try {
            PreparedStatement st = connection.prepareStatement("select password from users where user = ?");
            st.setString(1, user);
            rs = st.executeQuery();
          
            creat = rs.next();
            
        } catch (SQLException ex) {
            creat = false;
        }
        return creat;
    }
    
    /**
     * Checks if exists any user
     * 
     * @return True if exists at least one user
     */
    public boolean checkSomeUser() {
        boolean creat;
        ResultSet rs; 
        try {
            Statement st = connection.createStatement();
            
            rs = st.executeQuery("select password from users");
          
            creat = rs.next();
            
        } catch (SQLException ex) {
            creat = false;
        }
        return creat;
    }
    
    /**
     * Makes the list of all accounts of this user
     * 
     * @param userId the owner user identificator
     * @return the resultset with the descriptions
     * @throws java.lang.Exception
     */
    public ArrayList<Account> getAccountsData(int userId) throws Exception {
        ArrayList<Account> llista = new ArrayList();

        PreparedStatement st = connection.prepareStatement("select * from accounts where owner = ?");
        st.setInt(1, userId);
        ResultSet rs = st.executeQuery();

        while (rs.next()) {
            Account element = new Account(rs.getInt(1), rs.getString(2), rs.getString(3), rs.getString(4), rs.getString(5));
            llista.add(element);
        }
        rs.close();

        return llista;

    }
    
    /**
     * Creates an account to the database
     * 
     * @param ac the account to create
     * @param userId the owner user identificator
     * @return the DataBase index of the account
     * @throws java.lang.Exception
     */
    public int createAccount(Account ac, int userId) throws Exception {
        int id;
        PreparedStatement st = connection.prepareStatement("insert into accounts values (?,?,?,?,?,?);");
        PreparedStatement index = connection.prepareStatement("select account_id from accounts where account_id = (select max(account_id) from accounts);");
        
        st.setString(2, ac.getUser());
        st.setString(3, ac.getPassword());
        st.setString(4, ac.getDescripcio());
        st.setString(5, ac.getUrl());
        st.setInt(6, userId);
        connection.setAutoCommit(false);
        st.execute();
        ResultSet rs = index.executeQuery();
        id = rs.getInt("account_id");
        connection.setAutoCommit(true);
        
        return id;

    }
    
    /**
     * Updates an account to the database
     * 
     * @param ac the account to update
     * @throws java.lang.Exception
     */
    public void updateAccount(Account ac) throws Exception {

        PreparedStatement st = connection.prepareStatement("update accounts set user=?, password=?, descripcio=?, url=? where account_id=?");
        
        st.setString(1, ac.getUser());
        st.setString(2, ac.getPassword());
        st.setString(3, ac.getDescripcio());
        st.setString(4, ac.getUrl());
        st.setInt(5, ac.getId());
        connection.setAutoCommit(false);
        st.execute();
        connection.setAutoCommit(true);


    }
    
    /**
     * Remove an account to the database
     * 
     * @param ac the account to remove
     * @throws java.lang.Exception
     */
    public void deleteAccount(Account ac) throws Exception {

        PreparedStatement st = connection.prepareStatement("delete from accounts where account_id=?");
        
        st.setInt(1, ac.getId());
        
        connection.setAutoCommit(false);
        st.execute();
        connection.setAutoCommit(true);


    }
    
    /**
     * Deletes an user
     * 
     * @param user The user to delete
     * @throws Exception 
     */
    public void deleteUser(User user) throws Exception {
        ArrayList<Account> llista = getAccountsData(user.getId());
        
        Iterator it = llista.iterator();
        
        while (it.hasNext()) {
            Account element = (Account) it.next();
            deleteAccount(element);
        }
        
        PreparedStatement st = connection.prepareStatement("delete from users where user_id=?");
        st.setInt(1, user.getId());
        
        connection.setAutoCommit(false);
        st.execute();
        connection.setAutoCommit(true);
    }
    
}
