/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package uoc.carlesfigueras.passwordvault;

import java.security.SecureRandom;

/**
 * This class represents a Password.
 * It can generate a random password and check how strong
 * a password is
 * 
 * @author Carles Figueras Garcia
 */
public class Password {
    private static String lCase = "abcdefghijklmnopqrstuvwxyz";
    private static String uCase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    private static String sChar = "!@#$%^&*";
    private static String intChar = "0123456789";
    private SecureRandom r = new SecureRandom();
    private String password = "";
    
    /**
     * Constructor
     * 
     * @param length the length of the password to construct
     */
    public Password(int length) {
        int index;
        
        while (password.length () != length){
            int rPick = r.nextInt(4);
            if (rPick == 0){
                index = r.nextInt(25);
                password += lCase.charAt(index);
            } else if (rPick == 1) {
                index = r.nextInt (25);
                password += uCase.charAt(index);
            } else if (rPick == 2) {
                index = r.nextInt (7);
                password += sChar.charAt(index);
            } else if (rPick == 3){
                index = r.nextInt (9);
                password += intChar.charAt (index);
            }
        }
    }
    
    /**
     * Checks the entropy of the given password
     * 
     * @param s the password to check
     * @return the entropy in bits
     */
    public int checkStrength(String s) {
        int averageSymbols = 0;
        int i =  0;
        boolean lower = false;
        boolean upper = false;
        boolean symbol = false;
        boolean number = false;
        

        while ((!lower || !upper || !symbol || !number) && (i < s.length())) {
            if (isLower(s.substring(i, i+1)) && !lower) {
                lower = true;
                averageSymbols += 26;
            }
            if (isUpper(s.substring(i, i+1)) && !upper) {
                upper = true;
                averageSymbols += 26;
            }
            if (isSymbol(s.substring(i, i+1)) && !symbol) {
                symbol = true;
                averageSymbols += 8;
            }
            if (isNumber(s.substring(i, i+1)) && !number) {
                number = true;
                averageSymbols += 10;
            }
            i++;
        }
        
        double strength = s.length() * (Math.log(averageSymbols)/Math.log(2));
        
        return (int) Math.round(strength);
    }
    
    /**
     * Check if the string contains a lowercase
     * 
     * @param s the string to check
     * @return true if contains a lowercase
     */
    private boolean isLower (String s) {
        return lCase.contains(s);
    }
    
    /**
     * Check if the string contains a uppercase
     * 
     * @param s the string to check
     * @return true if contains a uppercase
     */
    private boolean isUpper (String s) {
        return uCase.contains(s);
    }
    
    /**
     * Check if the string contains any symbol of the sChar string
     * 
     * @param s the string to check
     * @return true if contains a symbol
     */
    private boolean isSymbol (String s) {
        return sChar.contains(s);
    }
    
    /**
     * Check if the string contains a number
     * 
     * @param s the string to check
     * @return true if contains a number
     */
    private boolean isNumber (String s) {
        return intChar.contains(s);
    }
    
    /**
     * The representation of the password
     * 
     * @return the password string
     */
    @Override
    public String toString() {
        return password;
    }
}
