/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package uoc.carlesfigueras.passwordvault;


import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.util.encoders.Hex;

/**
 * This class represents the user owner of the data.
 * It contains the password needed to access to all data and
 * is contained in a hash form and this class offers the possibility
 * to check and validate any password
 * 
 * @author Carles Figueras Garcia
 */
public class User {
    /* Variables */
    private int userId;
    private String username;
    private String salt;
    private String passwordHash;
    private String encryptionKey;
    
    
    /**
     * Constructor
     * 
     * @param user the username for this User
     * @param password the password for this User
     */
    public User(String user, String password) {
        this.username = user;
        this.salt = generateCondiment();
        this.encryptionKey = password;
        this.passwordHash = hashPassword(password.concat(salt));
    }
    
    /**
     * Constructor
     * 
     * @param user the username for this User
     * @param password the password for this User
     */
    public User(String user, char[] password) {
        StringBuffer cadena = new StringBuffer();
        int i;
        
        for (i=0; i<password.length; i++) {
            cadena.append(password[i]);
        }
        
        this.username = user;
        this.salt = generateCondiment();
        this.encryptionKey = cadena.toString();
        this.passwordHash = hashPassword(cadena.toString().concat(salt));
    }
    
    /**
     * Constructor
     *
     * @param user the username for this User
     * @param passwordHash the password in hash form
     * @param salt the salt used to create this hash
     */
    public User(String user, String passwordHash, String salt) {
        this.username = user;
        this.passwordHash = passwordHash;
        this.salt = salt;
    }
    
    /**
     * Constructor
     * 
     * @param user the username for this User
     * @param password the password for this User
     * @param salt the salt used to create the hash
     */
    public User(String user, char[] password, String salt) {
        StringBuffer cadena = new StringBuffer();
        int i;
        
        for (i=0; i<password.length; i++) {
            cadena.append(password[i]);
        }
        
        this.username = user;
        this.passwordHash = hashPassword(cadena.toString().concat(salt));
    }
    
    /**
     * Generates a random salt to use with the password
     * 
     * @return the salt generated
     */
    private String generateCondiment() {
        Password ret = new Password(8);
        return ret.toString();
    }

    
    /**
     * Check if password had the same hash to this
     * 
     * @param hash the password to check
     * @return 0 if the passwords had the same hash
     */
    public int checkPassword(String hash) {
        return this.passwordHash.compareTo(hash);
    }
    
    /**
     * Encode the password in the SHA512 hash algorithm
     * 
     * @param password the password to encode
     * @return a string with the hex representation of this hash
     */
    private String hashPassword(String password) {
        byte [] digesta = null;
        
        try {
            Security.addProvider(new BouncyCastleProvider());
            
            MessageDigest mda = MessageDigest.getInstance("SHA-512");

            mda.update(password.getBytes());

            digesta = mda.digest();
   
        } catch (NoSuchAlgorithmException ex) {
            Logger.getLogger(User.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        return Hex.toHexString(digesta);
    }
    
    /**
     * Get the salt
     * 
     * @return the salt used to encode this password
     */
    public String getSalt() {
        return salt.toString();
    }
    
    /**
     * Get the key needed to encrypt/decrypt data
     * 
     * @return the key
     */
    public String getKey() {
        return encryptionKey;
    }
    
    /**
     * Set the key needed to encrypt/decrypt data
     * 
     * @param key the key to set
     */
    public void setKey(String key) {
        encryptionKey = key;
    }
    
    /**
     * Gets the username of this user
     * 
     * @return the username
     */
    public String getUser() {
        return username;
    }
    
    /**
     * Set the id for this user
     * 
     * @param i the user id
     */
    public void setId(int i) {
        userId = i;
    }
    
    /**
     * Get the id of this user
     * 
     * @return the user id
     */
    public int getId() {
        return userId;
    }
    
    /**
     * Blank the current user information
     */
    public void flush() {
        username = "";
        encryptionKey = "";
        passwordHash = "";
        salt = "";
    }
    /**
     * The String representation of this password
     * 
     * @return the string
     */
    @Override
    public String toString() {
        return passwordHash;
    }

}
